@@ -170,11 +170,6 @@ class SqliteDocumentPersistence(object):
 
             self.cursor.executemany(CONTENT_NODE_PART_INSERT, cn_parts_values)
 
-        # for child in node.get_children():
-        #     child._parent = node
-        #     child._parent_uuid = node.uuid
-        #     self.__insert_node(child, node)
-
     def __clean_none_values(self, d):
         clean = {}
         for k, v in d.items():
@@ -274,6 +269,15 @@ class SqliteDocumentPersistence(object):
             children.append(self.__build_node(child_node))
         return children
 
+    def get_child_ids(self, content_node):
+
+        # We need to get the child nodes
+        children = []
+        for child_node in self.cursor.execute("select id, pid, nt, idx from cn where pid = ? order by idx",
+                                              [content_node.uuid]).fetchall():
+            children.append(child_node[0])
+        return children
+
     def get_node(self, node_id):
         node_row = self.cursor.execute("select id, pid, nt, idx from cn where id = ?", [node_id]).fetchone()
         if node_row:
@@ -354,13 +358,28 @@ class SqliteDocumentPersistence(object):
                               part if not isinstance(part, str) else None])
         self.cursor.executemany(CONTENT_NODE_PART_INSERT, all_parts)
 
-    def remove_content_node(self, child):
-        self.cursor.execute("delete from cnp where cn_id=?", [child.uuid])
-        self.cursor.execute("delete from cn where id=?", [child.uuid])
+    def remove_content_node(self, node):
+        def get_all_node_ids(node):
+            all_node_ids = []
+            if not node.virtual:
+                all_node_ids.append([node.uuid])
+                for child in node.get_children():
+                    all_node_ids.extend(get_all_node_ids(child))
+
+            return all_node_ids
+
+        all_child_ids = get_all_node_ids(node)
+
+        self.cursor.executemany("delete from cnp where cn_id=?", all_child_ids)
+        self.cursor.executemany("delete from cn where id=?", all_child_ids)
+        self.cursor.executemany("delete from f where cn_id=?", all_child_ids)
 
     def remove_all_features(self, node):
         self.cursor.execute("delete from f where cn_id=?", [node.uuid])
 
+    def remove_all_features_by_id(self, node_id):
+        self.cursor.execute("delete from f where cn_id=?", [node_id])
+
     def get_next_node_id(self):
         next_id = self.cursor.execute("select max(id) from cn").fetchone()
         if next_id[0] is None:
@@ -402,7 +421,9 @@ class SimpleObjectCache(object):
     def get_dirty_objs(self):
         results = []
         for id in self.dirty_objs:
-            results.append(self.get_obj(id))
+            node = self.get_obj(id)
+            if node is not None:
+                results.append(node)
         return results
 
     def undirty(self, obj):
@@ -421,6 +442,7 @@ class PersistenceManager(object):
         self.document = document
         self.node_cache = SimpleObjectCache()
         self.child_cache = {}
+        self.child_id_cache = {}
         self.feature_cache = {}
         self.content_parts_cache = {}
         self.node_parent_cache = {}
@@ -437,20 +459,16 @@ class PersistenceManager(object):
 
     def get_bytes(self):
 
-        # TODO we need to go through an flush anything dirty in the cache
-        # back to the persistence layer
-
         for node in self.node_cache.get_dirty_objs():
             if not node.virtual:
                 self._underlying_persistence.add_content_node(node, None)
-                self.node_cache.undirty(node)
-
-        for node_id, features in self.feature_cache.items():
-            node = self.get_node(node_id)
-            if not node.virtual:
                 self._underlying_persistence.remove_all_features(node)
-                for feature in features:
-                    self._underlying_persistence.add_feature(node, feature)
+                if node.uuid in self.feature_cache:
+                    for feature in self.feature_cache[node.uuid]:
+                        self._underlying_persistence.add_feature(node, feature)
+                if node.uuid in self.content_parts_cache:
+                    self._underlying_persistence.update_content_parts(node, self.content_parts_cache[node.uuid])
+                self.node_cache.undirty(node)
 
         return self._underlying_persistence.get_bytes()
 
@@ -464,32 +482,38 @@ class PersistenceManager(object):
 
         if parent:
             node._parent_uuid = parent.uuid
+            self.node_cache.add_obj(parent)
 
         self.node_cache.add_obj(node)
 
-        update_child_cache=False
+        update_child_cache = False
 
         if node.uuid not in self.node_parent_cache:
             self.node_parent_cache[node.uuid] = node._parent_uuid
-            update_child_cache=True
+            update_child_cache = True
 
         if node.uuid in self.node_parent_cache and node._parent_uuid != self.node_parent_cache[node.uuid]:
-
             # Remove from the old parent
+            self.child_id_cache[self.node_parent_cache[node.uuid]].remove(node.uuid)
             self.child_cache[self.node_parent_cache[node.uuid]].remove(node)
-
             # Add to the new parent
             self.node_parent_cache[node.uuid] = node._parent_uuid
-            update_child_cache=True
+            update_child_cache = True
 
         if update_child_cache:
 
             if node._parent_uuid not in self.child_cache:
                 self.child_cache[node._parent_uuid] = [node]
+                self.child_id_cache[node._parent_uuid] = set([node.uuid])
             else:
-                if node not in self.child_cache[node._parent_uuid]:
-                    self.child_cache[node._parent_uuid].append(node)
-                    self.child_cache[node._parent_uuid] = sorted(self.child_cache[node._parent_uuid], key=lambda x: x.index)
+                if node.uuid not in self.child_id_cache[node._parent_uuid]:
+                    self.child_id_cache[node._parent_uuid].add(node.uuid)
+                    current_children = self.child_cache[node._parent_uuid]
+                    if len(current_children) == 0 or node.index >= current_children[-1].index:
+                        self.child_cache[node._parent_uuid].append(node)
+                    else:
+                        self.child_cache[node._parent_uuid].append(node)
+                        self.child_cache[node._parent_uuid] = sorted(self.child_cache[node._parent_uuid], key=lambda x: x.index)
 
     def get_node(self, node_id):
 
@@ -498,6 +522,10 @@ class PersistenceManager(object):
             node = self._underlying_persistence.get_node(node_id)
             if node is not None:
                 self.node_cache.add_obj(node)
+                if node._parent_uuid:
+                    self.node_parent_cache[node.uuid] = node._parent_uuid
+                    if node._parent_uuid not in self.child_id_cache:
+                        self.get_node(node._parent_uuid)
 
         return node
 
@@ -505,8 +533,9 @@ class PersistenceManager(object):
 
         self.node_cache.remove_obj(node)
 
-        if node._parent_uuid is not None:
-            self.child_cache[node._parent_uuid].remove(node)
+        if node.uuid in self.node_parent_cache:
+            self.child_cache[self.node_parent_cache[node.uuid]].remove(node)
+            self.child_id_cache[self.node_parent_cache[node.uuid]].remove(node.uuid)
 
         self.content_parts_cache.pop(node.uuid, None)
         self.feature_cache.pop(node.uuid, None)
@@ -514,14 +543,25 @@ class PersistenceManager(object):
         self._underlying_persistence.remove_content_node(node)
 
     def get_children(self, node):
+        if node.uuid not in self.child_id_cache:
+            child_ids = self._underlying_persistence.get_child_ids(node)
+        else:
+            child_ids = self.child_id_cache[node.uuid]
+
         if node.uuid not in self.child_cache:
-            children = self._underlying_persistence.get_children(node)
-            self.child_cache[node.uuid] = sorted(children, key=lambda x: x.index)
+            new_children = []
+
+            for child_id in child_ids:
+                child_node = self.node_cache.get_obj(child_id)
+                new_children.append(child_node) if child_node is not None else new_children.append(
+                    self.get_node(child_id))
+
+            self.child_cache[node.uuid] = sorted(new_children, key=lambda x: x.index)
+            self.child_id_cache[node.uuid] = set(child_ids)
+
         return self.child_cache[node.uuid]
 
     def update_content_parts(self, node, content_parts):
-        if node.uuid is None:
-            return
         self.content_parts_cache[node.uuid] = content_parts
 
     def get_content_parts(self, node):
